﻿

Enumeration
	#Window
	#CanvasGadget
	#InputGadget
	#ErrorGadget
EndEnumeration


XIncludeFile "../Library/Lizard.pbi"


Procedure Callback_Error(Text.s)  ; Define a callback function for errors with the content "Text.s"
	SetGadgetText(#ErrorGadget, "Error: "+Text)
EndProcedure


; Procedure to plot a function f(x) in a canvas gadget.
Procedure EvaluateAndPlot(Function.s)
	
	Protected Time.i = ElapsedMilliseconds()
	Protected *Expression
	Protected Length.i, Dim Doubles.d(4)
	Protected I.i, Start.i = #True
	
	If Not Lizard::Evaluate("f(x?) := "+Function) = "Null"
		ProcedureReturn #False
	EndIf
	
	Lizard::Evaluate(~"list   = Iterate( f(x)//Calculate, {x, -5.0, 5.0, 0.02} );")  ; Iterate over f(x) from x = -5 to 5 in steps of 0.02
	Lizard::Evaluate(~"buffer = Apply(Join, ToString(If(Head(#)===Real,#,Overflow), BinaryFormat->\"float64\") &/@ list );")  ; convert the list of reals into an binary buffer (array of doubles)
	Length = Val(Lizard::Evaluate("list // Length"))
	Dim Doubles.d(Length-1)
	Lizard::GetMemory("buffer", @Doubles())
	
	SetGadgetText(#ErrorGadget, "No error: "+Str(ElapsedMilliseconds()-Time)+" ms")
	
	If StartVectorDrawing(CanvasVectorOutput(#CanvasGadget))
		VectorSourceColor($FFFFFFFF)
		FillVectorOutput()
		VectorSourceColor($FF404040)
		MovePathCursor(250, 0) : AddPathLine(0, 500, #PB_Path_Relative)
		MovePathCursor(0, 250) : AddPathLine(500, 0, #PB_Path_Relative)
		For I = 0 To 500 Step 50
			MovePathCursor(250, I) : AddPathLine(-5, 0, #PB_Path_Relative)
			MovePathCursor(I, 250) : AddPathLine(0, 5, #PB_Path_Relative)
		Next
		StrokePath(2)
		VectorSourceColor($FFC08040)
		For I = 0 To Length-1
			If Abs(Doubles(I)) < 1e4
				If Start : MovePathCursor(500.0*I/(Length-1), 250.0-50*Doubles(I)) : Start = #False
				Else :     AddPathLine(500.0*I/(Length-1), 250.0-50*Doubles(I))
				EndIf
			Else
				Start = #True
			EndIf
		Next
		StrokePath(4)
		StopVectorDrawing()
	EndIf
	
EndProcedure



; Lizard always has to be initialized first.
If Lizard::Initialize("../Library")  ;  Define here the path of the Lizard.dll location.
	Debug "Lizard kernel has been initialized. Version "+Lizard::Version()
Else
	Debug "Lizard kernel initialization failed!"
	End
EndIf


Lizard::BindCallback(Lizard::#Event_Error, @Callback_Error())  ; Bind the error callback.


; Open a window with a status bar and gadgets
OpenWindow(#Window, 0, 0, 500, 540, "Lizard - Plot Functions Example", #PB_Window_MinimizeGadget|#PB_Window_ScreenCentered)
CanvasGadget(#CanvasGadget, 0, 0, WindowWidth(#Window), WindowHeight(#Window)-40)
TextGadget(#PB_Any, 5, WindowHeight(#Window)-40, 40, 20, "f(x) :=")
StringGadget(#InputGadget, 45, WindowHeight(#Window)-40, WindowWidth(#Window)-45, 20, "3*Exp(-x^2/4)*Cos(7*x)")
StringGadget(#ErrorGadget, 0, WindowHeight(#Window)-20, WindowWidth(#Window)-45, 20, "", #PB_String_ReadOnly)

EvaluateAndPlot(GetGadgetText(#InputGadget))

; Event loop
Repeat
	Select WaitWindowEvent()
		Case #PB_Event_CloseWindow
			Break
		Case #PB_Event_Gadget
			Select EventGadget()
				Case #InputGadget
					EvaluateAndPlot(GetGadgetText(#InputGadget))
			EndSelect
	EndSelect
ForEver


; Quit Lizard.
Lizard::Terminate()


; IDE Options = PureBasic 6.00 LTS (Windows - x64)
; CursorPosition = 16
; Folding = -
; EnableXP
; DPIAware
; EnableCompileCount = 295
; EnableBuildCount = 0